package pojo;

import com.fasterxml.jackson.annotation.JsonIgnore;
import constant.CommonConstant;
import lombok.Data;
import utils.JsonUtils;

import java.util.Objects;

/**
 * @author songfuqiang
 * @description app返回包装类
 */
@Data
public class AppResultMap<T> {

    private int code;
    private String msg;
    private String appCode;
    private T data;

    public AppResultMap() {
        code = CommonConstant.SUCCESS;
        msg = CommonConstant.RESULT_SUCCESS;
        appCode = CommonConstant.APP_CODE;
    }

    public AppResultMap(int code, String msg) {
        this.code = code;
        this.msg = msg;
        this.appCode = CommonConstant.APP_CODE;
    }

    public AppResultMap(int code, String msg, T data) {
        super();
        this.code = code;
        this.msg = msg;
        this.appCode = CommonConstant.APP_CODE;
        this.data = data;
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMsg() {
        return msg;
    }

    public String getAppCode() {
        return appCode;
    }

    public void setAppCode(String appCode) {
        this.appCode = appCode;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        AppResultMap<?> resultMap = (AppResultMap<?>) o;
        return code == resultMap.code &&
                Objects.equals(msg, resultMap.msg) &&
                Objects.equals(data, resultMap.data);
    }

    @Override
    public int hashCode() {
        return Objects.hash(code, msg, data);
    }


    @Override
    public String toString() {
        return JsonUtils.getJsonString(this);
    }

    @JsonIgnore
    public boolean isSuccess() {
        return this.code == CommonConstant.SUCCESS;
    }

}
