package utils;

import constant.CommonConstant;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;

public class AesUtils {

    private static final Logger LOGGER = LoggerFactory.getLogger(AesUtils.class);

    private static final String ALGORITHM = "AES/CBC/PKCS5Padding";

    private static final String STRIP_STRING = " \0";

    private static Cipher encryptCipher;

    private static Cipher decryptCipher;

    public synchronized static String encrypt(String content) {
        return encrypt(CommonConstant.DEFAULT_ENCRYPT_KEY, CommonConstant.DEFAULT_ENCRYPT_IV_KEY, content);
    }

    public synchronized static String encrypt(String keyString, String ivKeyString, String content) {
        if (StringUtils.isBlank(content)) {
            return CommonConstant.STRING_EMPTY;
        }
        String encryptText = null;
        try {
            Key key = new SecretKeySpec(keyString.getBytes(StandardCharsets.US_ASCII), "AES");
            encryptCipher = Cipher.getInstance(ALGORITHM);
            encryptCipher.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(ivKeyString.getBytes(StandardCharsets.US_ASCII)));
        } catch (InvalidKeyException e) {
            LOGGER.error("[AesUtils] invalid key " + keyString, e);
        } catch (InvalidAlgorithmParameterException e) {
            LOGGER.error("[AesUtils] invalid algorithm param " + ivKeyString, e);
        } catch (NoSuchPaddingException | NoSuchAlgorithmException e) {
            LOGGER.error("[AesUtils] init cipher error", e);
        }
        try {
            byte[] encryptBytes = encryptCipher.doFinal(extendKey(content.getBytes(StandardCharsets.UTF_8)));
            encryptText = parseByte2Hex(encryptBytes);
        } catch (Exception e) {
            LOGGER.error("[AesUtils] encrypt error ", e);
        }
        return encryptText;
    }

    public synchronized static String decrypt(String content) {
        return decrypt(CommonConstant.DEFAULT_ENCRYPT_KEY, CommonConstant.DEFAULT_ENCRYPT_IV_KEY, content);
    }

    public synchronized static String decrypt(String keyString, String ivKeyString, String content) {
        if (StringUtils.isBlank(content)) {
            return CommonConstant.STRING_EMPTY;
        }
        String decryptText = null;
        try {
            Key key = new SecretKeySpec(keyString.getBytes(StandardCharsets.US_ASCII), "AES");
            IvParameterSpec ivKey = new IvParameterSpec(ivKeyString.getBytes(StandardCharsets.US_ASCII));
            decryptCipher = Cipher.getInstance(ALGORITHM);
            decryptCipher.init(Cipher.DECRYPT_MODE, key, ivKey);
        } catch (InvalidKeyException e) {
            LOGGER.error("[AesUtils]invalid key" + keyString, e);
        } catch (InvalidAlgorithmParameterException e) {
            LOGGER.error("[AesUtils] encoding unsupported " + keyString, e);
        } catch (NoSuchPaddingException | NoSuchAlgorithmException e) {
            LOGGER.error("[AesUtils] init cipher error", e);
        }
        try {
            byte[] encryptBytes = parseHex2Byte(content);
            byte[] decryptBytes = decryptCipher.doFinal(encryptBytes);
            decryptText = new String(decryptBytes, StandardCharsets.UTF_8);
            decryptText = StringUtils.strip(decryptText, STRIP_STRING);
        } catch (IllegalBlockSizeException | BadPaddingException e) {
            LOGGER.error("[AesUtils]invalid key " + keyString, e);
        }
        return decryptText;
    }

    private static byte[] parseHex2Byte(String hexText) {
        if (StringUtils.isEmpty(hexText)) {
            return null;
        }
        byte[] result = new byte[hexText.length() / 2];
        for (int i = 0; i < hexText.length() / 2; i++) {
            int high = Integer.parseInt(hexText.substring(i * 2, i * 2 + 1), 16);
            int low = Integer.parseInt(hexText.substring(i * 2 + 1, i * 2 + 2), 16);
            result[i] = (byte) (high * 16 + low);
        }
        return result;
    }

    private static String parseByte2Hex(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        for (byte aByte : bytes) {
            String hex = Integer.toHexString(aByte & 0xFF);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }
            sb.append(hex.toLowerCase());
        }
        return sb.toString();
    }

    private static byte[] extendKey(byte[] input) {
        int rest = input.length % 16;
        if (rest > 0) {
            byte[] result = new byte[input.length + (16 - rest)];
            System.arraycopy(input, 0, result, 0, input.length);
            return result;
        }
        return input;
    }

    public static void main(String[] args) {
        System.out.println(encrypt("18627945022"));
        System.out.println(decrypt(CommonConstant.DEFAULT_ENCRYPT_KEY, CommonConstant.DEFAULT_ENCRYPT_IV_KEY, "54db89effcf52bb979778f718f75aa5493439afa76973581ceeee35150764066"));
    }

}
