package utils;

import org.redisson.api.RLock;

import java.util.concurrent.TimeUnit;

/**
 * 获取和释放分布式锁
 *
 * @author tangxiangan
 */
public interface DistributedLocker {

    /**
     * 获取锁 拿不到lock就不罢休，不然线程就一直block
     * @param lockKey
     * @return
     */
    RLock lock(String lockKey);

    /**
     *获取锁 带有时间
     * @param lockKey
     * @param timeout 加锁时间 单位为秒
     * @return
     */
    RLock lock(String lockKey, long timeout);

    /**
     * 获取锁 带有时间 时间单位有unit决定
     * @param lockKey
     * @param unit 加锁时间
     * @param timeout 时间单位
     * @return
     */
    RLock lock(String lockKey, TimeUnit unit, long timeout);

    /**
     *  带时间限制的获取锁，拿不到lock，就等一段时间，超时返回false.
     * @param lockKey
     * @param unit 时间单位
     * @param waitTime 等待时间
     * @param leaseTime 获取锁后加锁时间
     * @return
     */
    boolean tryLock(String lockKey, TimeUnit unit, long waitTime, long leaseTime);

    /**
     * 释放锁
     * @param lockKey
     */
    void unlock(String lockKey);

    /**
     * 释放锁
     * @param lock RLock类型
     */
    void unlock(RLock lock);

}
