package utils;


import com.github.pagehelper.PageInfo;
import constant.CommonConstant;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import pojo.PageVO;
import pojo.QueryParams;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.Collectors;

public class QueryUtils {

    public static String generateLikeString(String string){
        try {
            string = URLDecoder.decode(URLDecoder.decode(string, CommonConstant.ENCODING_UTF8), CommonConstant.ENCODING_UTF8);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("unsupport encoding");
        }
        return "%"+string+"%";
    }

    public static String generateLikeStringHalf(String string){
        try {
            //处理下%和_的问题，应该还有其他通配符
            string = string.replace(CommonConstant.STRING_PERCENT,"\\" + CommonConstant.STRING_PERCENT)
                    .replace(CommonConstant.STRING_UNDERLINE,"\\" + CommonConstant.STRING_UNDERLINE);
            string = URLDecoder.decode(URLEncoder.encode(string, CommonConstant.ENCODING_UTF8), CommonConstant.ENCODING_UTF8);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("unsupport encoding");
        }
        return string+"%";
    }



    public static <T extends QueryParams> void handlePageAndRow(T param) {
        param.setPage(PageUtils.getParamToInt(param.getPage(), CommonConstant.PAGENUM));
        param.setRows(PageUtils.getParamToInt(param.getRows(), CommonConstant.PAGESIZE));
    }

    public static <T extends QueryParams> void handleParam(T param) {
        Integer pageNum = PageUtils.getParamToInt(param.getPage(), CommonConstant.PAGENUM);
        Integer pageSize = PageUtils.getParamToInt(param.getRows(), CommonConstant.PAGESIZE);
        String orderByClause = PageUtils.getOrderByClause(param.getSidx(), param.getSord(), CommonConstant.ORDERBYCLAUSE);
        param.setOrderByClause(orderByClause);
        param.setPage(pageNum);
        param.setRows(pageSize);
    }

    public static <T,V> PageVO<T> convertPageInfoToVO(PageInfo<V> pageInfo, BeanConverter<V,T> converter){
        PageVO<T> pageVO = new PageVO<>();
        BeanUtils.copyProperties(pageInfo,pageVO);
        pageVO.setList(convertListToVO(pageInfo.getList(),converter));
        return pageVO;
    }


    public static <T,V> List<T> convertListToVO(List<V> list, BeanConverter<V,T> converter){
        List<T> voList = new CopyOnWriteArrayList<>();
        if(!CollectionUtils.isEmpty(list)){
            voList = list.stream().map(converter::convert).collect(Collectors.toList());
        }
        return voList;
    }
}
