package utils;


import constant.CommonConstant;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import pojo.MobSendResult;
import pojo.YunpianSendResult;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Component
@Slf4j
public class SMSUtils {

    @Value("${mob.sms.apikey:}")
    private String SMSApiKey;

    @Value("${mob.sms.sendUrl:}")
    private String SendSMSUrl;

    @Value("${mob.sms.verifyUrl:}")
    private String VerifySMSUrl;


    public boolean sendSMS(List<String> phoneList) {
        String phoneListStr = StringUtils.join(phoneList, CommonConstant.STRING_SIGN_COMMA);
        Map<String, String> params = new HashMap<>();
        params.put("appkey", SMSApiKey);
        params.put("phone", phoneListStr);
        params.put("zone", "86");
        String response = HttpClientUtils.doPost(SendSMSUrl, params);
        MobSendResult mobSendResult = JsonUtils.jsonStringToBean(response, MobSendResult.class);
        if(mobSendResult.getStatus().equals(CommonConstant.SUCCESS)){
            return Boolean.TRUE;
        } else {
            log.error("sendSMS.error:"+response);
            return Boolean.FALSE;
        }
    }

    public boolean verifySMS(List<String> phoneList,String code) {
        String phoneListStr = StringUtils.join(phoneList, CommonConstant.STRING_SIGN_COMMA);
        Map<String, String> params = new HashMap<>();
        params.put("appkey", SMSApiKey);
        params.put("phone", phoneListStr);
        params.put("zone", "86");
        params.put("code", code);
        String response = HttpClientUtils.doPost(VerifySMSUrl, params);
        log.info("返回信息：" + response);
        MobSendResult mobSendResult = JsonUtils.jsonStringToBean(response, MobSendResult.class);
        if(mobSendResult.getStatus().equals(CommonConstant.SUCCESS)){
           return Boolean.TRUE;
        }
        else {
            return Boolean.FALSE;
        }
    }


    public static String delHTMLTag(String htmlStr) {
        String regEx_script = "<script[^>]*?>[\\s\\S]*?<\\/script>"; //定义script的正则表达式
        String regEx_style = "<style[^>]*?>[\\s\\S]*?<\\/style>"; //定义style的正则表达式
        String regEx_html = "<[^>]+>"; //定义HTML标签的正则表达式

        Pattern p_script = Pattern.compile(regEx_script, Pattern.CASE_INSENSITIVE);
        Matcher m_script = p_script.matcher(htmlStr);
        htmlStr = m_script.replaceAll(""); //过滤script标签

        Pattern p_style = Pattern.compile(regEx_style, Pattern.CASE_INSENSITIVE);
        Matcher m_style = p_style.matcher(htmlStr);
        htmlStr = m_style.replaceAll(""); //过滤style标签

        Pattern p_html = Pattern.compile(regEx_html, Pattern.CASE_INSENSITIVE);
        Matcher m_html = p_html.matcher(htmlStr);
        htmlStr = m_html.replaceAll(""); //过滤html标签

        return htmlStr.trim(); //返回文本字符串
    }

    public static void main(String[] args) {
    }
}
