package cn.kdan.pdf.backend.core.config;

import constant.CommonConstant;
import exception.BackendRuntimeException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import pojo.ResultMap;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author tangxiangan
 */
@ControllerAdvice
public class ControllerExceptionHandler {
    private final Logger logger = LoggerFactory.getLogger(ControllerExceptionHandler.class);

    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseBody
    public ResultMap<?> handlerArgumentCheck(MethodArgumentNotValidException ex) {
        BindingResult bindingResult = ex.getBindingResult();
        List<FieldError> fieldErrorList = bindingResult.getFieldErrors();

        return new ResultMap<>(CommonConstant.EXCEPTION_CODE_PARAMETERS_ERROR,
                fieldErrorList.stream().map(DefaultMessageSourceResolvable::getDefaultMessage).collect(Collectors.joining(";")));
    }

    @ExceptionHandler(BackendRuntimeException.class)
    @ResponseBody
    public ResultMap<?> handlerDevOpsRuntimeException(BackendRuntimeException e) {
        return new ResultMap<>(CommonConstant.EXCEPTION_CODE_RUNTIME_ERROR, e.getMessage());
    }

    @ExceptionHandler(ConstraintViolationException.class)
    @ResponseBody
    public ResultMap<?> handlerConstraintViolationException(ConstraintViolationException e) {
        return new ResultMap<>(CommonConstant.EXCEPTION_CODE_RUNTIME_ERROR,
                e.getConstraintViolations().stream().map(ConstraintViolation::getMessage).collect(Collectors.joining(";")));
    }

    @ExceptionHandler(Exception.class)
    @ResponseBody
    public ResultMap<?> handlerException(Exception e) {
        logger.info(e.getMessage(), e);
        return new ResultMap<>(CommonConstant.EXCEPTION_CODE_SERVER_ERROR, "错误信息： " + e.getMessage());
    }


    @ExceptionHandler(MissingServletRequestParameterException.class)
    @ResponseBody
    public ResultMap<?> handlerMissingServletRequestParameterException(MissingServletRequestParameterException e) {
        logger.info(e.getMessage(), e);
        return new ResultMap<>(CommonConstant.EXCEPTION_CODE_SERVER_ERROR, "参数错误： " + e.getParameterName() + " 缺失");
    }

}
