package cn.kdan.pdf.backend.core.config.oauth;

import cn.kdan.pdf.backend.core.constant.AuthConstant;
import cn.kdan.pdf.backend.core.pojo.oauth2.UserInfo;
import cn.kdan.pdf.backend.core.service.AuthService;
import cn.kdan.pdf.backend.core.service.UserService;
import constant.CommonConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.stereotype.Component;
import pojo.CustomUserDetails;
import pojo.ResultMap;
import utils.JsonUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.Principal;
import java.util.Collections;

/**
 * @author tangxiangan
 */
@Component
public class LoginAuthenticationSuccessHandler implements AuthenticationSuccessHandler {

    @Autowired
    private UserService userService;
    @Autowired
    private AuthService authService;

    @Override
    public void onAuthenticationSuccess(HttpServletRequest request, HttpServletResponse response,
                                        Authentication authentication) throws IOException {
        //Authentication接口封装认证信息
        response.setContentType("application/json;charset=UTF-8");
        //将authentication认证信息转换为json格式的字符串写到response里面去
        PrintWriter writer = response.getWriter();
        String userId = ((CustomUserDetails) authentication.getPrincipal()).getId();
        //根据认证信息获取授权码
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(authentication.getPrincipal(), null, Collections.emptySet());

        String code = authService.getAuthorizeCode(userId, authenticationToken);
        //登录成功，返回授权码
        writer.write(JsonUtils.getJsonString(new ResultMap<>(CommonConstant.SUCCESS, AuthConstant.GET_AUTHORIZE_CODE_SUCCESS, code)));
        writer.flush();
        writer.close();
    }
}
