package cn.kdan.pdf.backend.core.controller;

import cn.kdan.pdf.backend.core.params.MissionCreateParam;
import cn.kdan.pdf.backend.core.pojo.MissionVO;
import cn.kdan.pdf.backend.core.pojo.comPdfKit.WebhookSendDTO;
import cn.kdan.pdf.backend.core.service.MissionService;
import constant.CommonConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;

import java.util.List;

@RestController
@RequestMapping("/mission")
public class MissionController {

    @Autowired
    private MissionService missionService;

    /**
     * 根据文件列表创建任务和插入文件信息（转档流程第一步）
     *
     * @param paramList MissionCreateParam
     * @return MissionPojo
     */
    @PostMapping("create")
    public ResultMap<MissionVO> create(@RequestBody List<MissionCreateParam> paramList) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, missionService.create(paramList));
    }

    /**
     * 根据任务id进行saas任务转档
     *
     * @param missionId 任务id
     * @return 任务id
     */
    @PostMapping("convertFile")
    public ResultMap<String> convertFile(String missionId) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, missionService.convertFile(missionId));
    }

    /**
     * 查询文件任务状态
     *
     * @param missionId 任务id
     * @return 文件状态信息
     */
    @PostMapping("queryFileStatus")
    public ResultMap<List<MissionVO>> queryFileStatus(String missionId) {
        List<MissionVO> result = missionService.queryFileStatus(missionId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, result);
    }

    /**
     * saas回调路径
     *
     * @param dto saas回调参数
     * @return 回调成功提示
     */
    @PostMapping("saasCallback")
    public ResultMap<String> saasCallback(@RequestBody WebhookSendDTO dto){
        missionService.saasCallback(dto);
        return new ResultMap<>(CommonConstant.SUCCESS,CommonConstant.RESULT_SUCCESS, "回调成功");
    }


}
