package cn.kdan.pdf.backend.core.controller;

import cn.kdan.pdf.backend.core.constant.MissionFilesConstant;
import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.pojo.MissionFileVO;
import cn.kdan.pdf.backend.core.service.MemberService;
import cn.kdan.pdf.backend.core.service.MissionFilesService;
import com.github.pagehelper.PageInfo;
import constant.CommonConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pojo.AppResultMap;
import pojo.ResultMap;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/missionFile")
public class MissionFilesController {
    @Autowired
    private MemberService memberService;
    @Autowired
    private MissionFilesService missionFilesService;

    /**
     * 官网用户转档记录分页查询
     *
     * @param page     页数
     * @param pageSize 页码
     * @return page
     */
    @GetMapping("page")
    public ResultMap<PageInfo<MissionFileVO>> page(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                   @RequestParam(required = false, defaultValue = "25") Integer pageSize) {
        Members members = memberService.getCurrentUser();
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, missionFilesService.page(page, pageSize, members.getId()));
    }

    /**
     * 删除转档记录
     *
     * @param ids ids
     * @return Boolean
     */
    @PostMapping("delete")
    public ResultMap<Boolean> delete(@NotBlank(message = "转档id不能为空") @RequestParam List<String> ids) {
        return new ResultMap<>(CommonConstant.SUCCESS, MissionFilesConstant.RETURN_MSG_DELETE_FILE_SUCCESS, missionFilesService.delete(ids));
    }


    /**
     * 上传文件
     *
     * @param file 文件
     * @param missionFileId 文件id
     * @return Boolean
     */
    @PostMapping("upload")
    public ResultMap<Boolean> upload(@NotBlank(message = "转档文件不能为空") @RequestParam(value="file") MultipartFile file,@NotBlank(message = "转档文件Id不能为空") @RequestParam String missionFileId) {
        return new ResultMap<>(CommonConstant.SUCCESS, MissionFilesConstant.RETURN_MSG_UPLOAD_FILE_SUCCESS, missionFilesService.upload(file,missionFileId));
    }

    /**
     * 附件下载
     *
     * @param param
     * @return ResponseDTO
     */
    @PostMapping(value = "/download")
    public void downloadFile(@RequestParam Map<String, Object> param, HttpServletResponse response) throws Exception {
        String url1 = param.get("url").toString();
        URL url = new URL(url1);
        URLConnection conn = url.openConnection();
        InputStream inputStream = conn.getInputStream();
        response.reset();
        response.setContentType(conn.getContentType());
        //纯下载方式 文件名应该编码成UTF-8
        String name = param.get("name").toString();
        response.setHeader("Content-Disposition",
                "attachment; filename=" + URLEncoder.encode(name, "UTF-8"));
        byte[] buffer = new byte[1024];
        int len;
        OutputStream outputStream = response.getOutputStream();
        while ((len = inputStream.read(buffer)) > 0) {
            outputStream.write(buffer, 0, len);
        }
        inputStream.close();
    }

    /**
     * 获取转档文件总数
     * @return 文件总数
     */
    @GetMapping("/getTotalConvertFile")
    public AppResultMap<Integer> getTotalConvertFile(){
        return new AppResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, missionFilesService.getTotalConvertFile());
    }
}
