package cn.kdan.pdf.backend.core.controller;

import cn.kdan.pdf.backend.core.pojo.OrderVO;
import cn.kdan.pdf.backend.core.service.OrderService;
import constant.CommonConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;

import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotBlank;

/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/order")
public class OrderController {
    @Autowired
    private OrderService orderService;

    /**
     * 查询微信订单信息
     * @param orderId 订单id
     * @return getWxOrder
     */
    @GetMapping("getWxOrder")
    public ResultMap<OrderVO> getWxOrders(@NotBlank(message = "订单Id不能为空")@RequestParam String orderId) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, orderService.getWxOrders(orderId));
    }

    /**
     * 微信订单回调接口
     * @param xmlData 微信返回的xml数据
     * @return 返回给微信确认的xml
     */
    @PostMapping("syncOrder")
    public ResultMap<String> syncOrder(@RequestBody String xmlData) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, orderService.syncOrder(xmlData));
    }

    /**
     * 支付宝订单回调接口
     *
     * @param request 支付宝回调参数
     * @return String
     */
    @RequestMapping("alipaySyncOrder")
    public ResultMap<String> alipaySyncOrder(HttpServletRequest request) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, orderService.alipaySyncOrder(request));
    }
}
