package cn.kdan.pdf.backend.core.service;

import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.model.member.MemberInfoResp;
import cn.kdan.pdf.backend.core.model.member.MemberRegisterReq;
import cn.kdan.pdf.backend.core.model.member.MemberRegisterResp;
import cn.kdan.pdf.backend.core.model.member.ModifyPasswordReq;
import cn.kdan.pdf.backend.core.params.BindAccountParam;
import cn.kdan.pdf.backend.core.params.CheckExistsParam;
import cn.kdan.pdf.backend.core.params.UserResetPwdParams;
import cn.kdan.pdf.backend.core.pojo.ExistsMemberVo;
import cn.kdan.pdf.backend.core.pojo.app.BindAccountVo;
import cn.kdan.pdf.backend.core.pojo.app.MemberInfoVo;
import cn.kdan.pdf.backend.core.pojo.app.MemberVo;

public interface MemberService {

    /**
     * 用户注册
     *
     * @param req 注册参数
     * @return 注册回参
     */
    MemberRegisterResp register(MemberRegisterReq req);

    /**
     * 修改密码
     *
     * @param req 修改密码请求参数
     */
    void modifyPassword(ModifyPasswordReq req);

    /**
     * 修改昵称
     *
     * @param name 修改昵称请求参数
     */
    void modifyNickname(String name);

    /**
     * 获取用户信息
     *
     * @param id 用户id，无id就传null
     * @return 用户信息
     */
    MemberInfoResp getMemberDetail(String id);

    /**
     * 获取当前用户
     *
     * @return Members
     */
    Members getCurrentUser();

    /**
     * 判断用户会员状态,并在会员过期后更新会员信息
     * @param memberId 用户id
     * @return 会员状态
     */
    Integer judgeSubscribeType(String memberId);

    /**
     * 根据id查询用户
     *
     * @param memberId 用户id
     * @return Members
     */
    Members getById(String memberId);

    /**
     * 重置密码
     *
     * @param params UserResetPwdParams
     * @return boolean
     */
    boolean resetPassword(UserResetPwdParams params);

    /**
     * 更新用户
     *
     * @param member 用户
     */
    void update(Members member);

    /**
     * 根据用户名查询用户
     * @param name 昵称
     * @return 用户信息
     */
    Members selectByName(String name);

    /**
     * 注销账号
     * @return 用户id
     */
    String cancelAccount();

    /**
     * 绑定账号
     * @return 绑定信息
     */
    BindAccountVo bindAccount(BindAccountParam param);

    /**
     * 解绑账号
     * @return 解绑信息
     */
    BindAccountVo unbindAccount(BindAccountParam param);

    /**
     * 修改用户昵称
     * @param name 用户输入新昵称
     * @return 用户信息
     */
    MemberVo appModifyName(String name);

    /**
     * 根据用户id删除用户
     * @param memberId 用户ID
     */
    void deleteByMemberId(String memberId);

    /**
     * app获取用户信息
     * @param subscription 1:不显示订阅信息 0:显示订阅信息
     * @param withSocial 如果是QQ、微信登录，传入字段可实现第三方登录的信息。0:不显示， 1:显示
     * @return 用户详细信息
     */
    MemberInfoVo appGetMemberInfo(String subscription, String withSocial);

    /**
     * 检查内容是否存在
     * @param param param
     * @return vo
     */
    ExistsMemberVo checkExists(CheckExistsParam param);

    /**
     * app端修改密码
     * @param param param
     * @return 是否成功
     */
    boolean appResetPassword(UserResetPwdParams param);
}
