package cn.kdan.pdf.backend.core.service;


import cn.kdan.pdf.backend.core.enums.MissionFileStatusEnum;
import cn.kdan.pdf.backend.core.model.MissionFiles;
import cn.kdan.pdf.backend.core.pojo.MissionFileVO;
import com.github.pagehelper.PageInfo;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 转档文件
 *
 * @author tangxiangan
 */
public interface MissionFilesService {


    /**
     * 分页查询用户的转档记录
     *
     * @param page     页码
     * @param pageSize 条数
     * @param memberId 用户id
     * @return PageInfo
     */
    PageInfo<MissionFileVO> page(Integer page, Integer pageSize, String memberId);

    /**
     * 删除
     *
     * @param ids id集合
     * @return Boolean
     */
    Boolean delete(List<String> ids);

    /**
     * 插入文件
     *
     * @param missionFile 文件对象
     */
    void insert(MissionFiles missionFile);

    /**
     * 上传文件
     *
     * @param file 文件
     * @param missionFileId  文件id
     * @return Boolean
     */
    Boolean upload(MultipartFile file, String missionFileId);

    /**
     * 根据id获取转档文件记录
     *
     * @param missionFileId missionFileId
     * @return MissionFiles
     */
    MissionFiles getById(String missionFileId);

    /**
     * 更新missionFile
     *
     * @param missionFiles 文件信息
     */
    void update(MissionFiles missionFiles);

    /**
     * 根据文件唯一标识查询
     *
     * @param processId 文件唯一标识id
     * @return 文件信息
     */
    MissionFiles queryByProcessId(String processId);

    /**
     * 根据saas任务id更新转档开始时间
     *
     * @param taskId saas任务id
     */
    void updateBySaasTaskId(String taskId);

    /**
     * 根据文件id（processId）修改状态
     *
     * @param processId 文件唯一标识
     * @param status 状态
     */
    void updateByProcessId(String processId, MissionFileStatusEnum status);

    /**
     * 根据SaasTaskId查询文件集合
     *
     * @param id saas任务id
     * @return 文件集合
     */
    List<MissionFiles> getBySaasId(String id);

    /**
     * 获取转档文件总数
     * @return 文件总数
     */
    Integer getTotalConvertFile();
}
