package cn.kdan.pdf.backend.core.service;

import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.model.Pricings;
import cn.kdan.pdf.backend.core.model.PricingDiscounts;
import cn.kdan.pdf.backend.core.model.pricing.QueryPricingReq;
import cn.kdan.pdf.backend.core.model.pricing.QueryPricingResp;
import cn.kdan.pdf.backend.core.params.QueryPricingParam;

import java.util.List;

public interface PricingService {

    /**
     * 查询券服务列表
     *
     * @param req 查询券服务参数
     * @return 券列表
     */
    QueryPricingResp getPricingList(QueryPricingReq req);

    /**
     * 根据活跃折扣计算时间
     *
     * @param item            SetPricings
     * @param pricingDiscount pricingDiscount
     * @return String
     */
    String countPeriod(Pricings item, PricingDiscounts pricingDiscount);

    /**
     * 根据获取券服务
     *
     * @param id 券服务id
     * @return 券服务对象
     */
    Pricings getById(Integer id);

    /**
     * 根据id获取在售的人民币类型的券服务
     *
     * @param pricingId id
     * @return List
     */
    List<Pricings> getForSaleRMBById(String pricingId);

    /**
     * 根据活跃折扣计算价格
     *
     * @param item 券服务对象
     * @param activeDiscount 活跃折扣
     * @param member 用户
     * @param client 客户端
     * @return  计算后的价格
     */
    float countPrice(Pricings item, PricingDiscounts activeDiscount, Members member, String client);

    /**
     * 计算剩余券数量
     *
     * @param item 券服务对象
     * @param pricingDiscount 活跃折扣
     * @return 券数量
     */
    Integer countPointsAmount(Pricings item, PricingDiscounts pricingDiscount);

    /**
     * app端获取商品价目表
     * @param param 入参
     * @return 商品价目表列表
     */
    QueryPricingResp appList(QueryPricingParam param);
}
