package cn.kdan.pdf.backend.core.service;


import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.model.PricingDiscounts;
import cn.kdan.pdf.backend.core.model.SetPricings;
import cn.kdan.pdf.backend.core.model.Subscriptions;
import cn.kdan.pdf.backend.core.pojo.SetPricingVO;

import java.util.List;

/**
 * 会员类型表
 *
 * @author tangxiangan
 */
public interface SetPricingService {


    /**
     * 获取所有上架的会员类型以及其对应折扣
     *
     * @param client web 或者 app
     * @param member 用户信息
     * @return List
     */
    List<SetPricingVO> list(String client,Members member);

    /**
     * 根据id获取在售的人民币类型的会员服务
     *
     * @param setPricingId id
     * @return List
     */
    List<SetPricings> getForSaleRMBById(String setPricingId);

    /**
     * 获取会员服务
     *
     * @param setPricingId id
     * @return SetPricings
     */
    SetPricings getById(Integer setPricingId);

    /**
     * 获取活跃折扣
     *
     * @param pricingDiscountId 折扣id
     * @param member            当前登录用户
     * @return PricingDiscounts
     */
    PricingDiscounts getActiveDiscount(String pricingDiscountId, Members member);

    /**
     * 计算剩余券数量
     *
     * @param item            SetPricings
     * @param pricingDiscount pricingDiscount
     * @return Integer
     */
    Integer countPointsAmount(SetPricings item, PricingDiscounts pricingDiscount);

    /**
     * 根据活跃折扣计算时间
     *
     * @param item            SetPricings
     * @param pricingDiscount pricingDiscount
     * @return String
     */
    String countPeriod(SetPricings item, PricingDiscounts pricingDiscount);

    /**
     * 根据活跃折扣计算价格
     *
     * @param item           SetPricings
     * @param activeDiscount PricingDiscounts折扣
     * @param member         用户
     * @param client         客户端
     * @return float
     */
    float countPrice(SetPricings item, PricingDiscounts activeDiscount, Members member, String client);

    /**
     * 查询mode为0的会员服务
     * @param memberId 用户id
     * @return Subscriptions
     */
    List<Subscriptions> selectMode0(String memberId);

    List<Subscriptions> selectPByModeAndMemberId(String memberId,String mode);
}
