package cn.kdan.pdf.backend.core.service;

import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.model.SocialAccounts;
import cn.kdan.pdf.backend.core.params.BindAccountParam;
import cn.kdan.pdf.backend.core.pojo.app.SocialAccountVo;

import java.util.List;

/**
 * @author : SongFuQiang
 * @date : 2022/12/9 16:37
 */
public interface SocialAccountService {

    /**
     * 根据第三方账号id查询社交账号
     * @param accountId 第三方账号id
     * @return 社交账号信息
     */
    SocialAccounts selectByAccountId(String accountId);

    /**
     * 插入社交账号表数据
     * @param socialAccounts socialAccounts
     * @return
     */
    int insertSocialAccount(SocialAccounts socialAccounts);

    /**
     * 根据用户id查询社交账号表数据
     * @param userId 用户id
     * @return 社交账号信息
     */
    List<SocialAccounts> selectByMemberId(String userId);

    /**
     * 根据用户id删除社交账号信息
     * @param memberId 用户ID
     */
    void deleteByMemberId(String memberId);

    /**
     * 绑定社交账号
     * @param member 当前用户
     * @param param 绑定参数
     * @param socialAccounts 社交账号返回信息
     */
    List<SocialAccountVo> bindSocialAccount(Members member, BindAccountParam param, List<SocialAccountVo> socialAccounts);

    /**
     * 解绑账号
     * @param currentUser 当前用户
     * @param provider 第三方
     */
    List<SocialAccountVo> unbindSocialAccount(Members currentUser, String provider, List<SocialAccountVo> list);
}
