package cn.kdan.pdf.backend.core.service;


import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.model.Subscriptions;
import cn.kdan.pdf.backend.core.params.ExpenseParam;
import cn.kdan.pdf.backend.core.pojo.SubscriptionVO;
import cn.kdan.pdf.backend.core.pojo.app.CreateSubVo;
import cn.kdan.pdf.backend.core.pojo.app.ExpenseVo;
import cn.kdan.pdf.backend.core.pojo.app.RemindVo;

/**
 * @author tangxiangan
 */
public interface SubscriptionService {


    /**
     * 计算用户对于某种折扣的使用次数
     *
     * @param userId
     * @param discountId
     * @return
     */
    long countTimesById(String userId, String discountId);


    /**
     * 用户选择会员产品后生成订阅和订单 微信支付下单
     *
     * @param client     客户端
     * @param payment    支付方式
     * @param targetType 订阅类型
     * @param targetId   目标id
     * @return SubscriptionPojo
     */
    SubscriptionVO create(String client, Integer payment, String targetType, String targetId);

    /**
     * 计算订阅的开始时间和结束时间
     *
     * @param force        true
     * @param client       web
     * @param subscription 订阅对象
     * @param member       用户
     * @param target       pricing SetPricing
     */
    void renewInfo(Boolean force, String client, Subscriptions subscription, Members member, Object target);

    /**
     * 更新
     *
     * @param subscription subscription
     */
    void update(Subscriptions subscription);

    /**
     * 获取单个对象
     *
     * @param subscriptionId 订阅id
     * @return Subscriptions
     */
    Subscriptions getById(String subscriptionId);

    /**
     * 获取套餐过期信息
     * @return 过期信息返回
     */
    RemindVo remindRenew();

    /**
     * 转换券消耗
     * @param param param
     * @return 券消耗信息返回
     */
    ExpenseVo expense(ExpenseParam param);

    /**
     * app端创建订单
     * @param client
     * @param payment
     * @param targetType
     * @param targetId
     * @return
     */
    CreateSubVo appCreate(String client, Integer payment, String targetType, String targetId);
}
