package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.enums.LanguageEnum;
import cn.kdan.pdf.backend.core.mapper.AdvertisementsMapper;
import cn.kdan.pdf.backend.core.model.Advertisements;
import cn.kdan.pdf.backend.core.model.AdvertisementsExample;
import cn.kdan.pdf.backend.core.params.QueryAdvertisementParam;
import cn.kdan.pdf.backend.core.pojo.app.AdvertisementVo;
import cn.kdan.pdf.backend.core.properties.OssProperties;
import cn.kdan.pdf.backend.core.service.AdvertisementService;
import cn.kdan.pdf.backend.core.utils.CommonBusinessUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * @author sfq
 * @description
 */
@Service
public class AdvertisementServiceImpl implements AdvertisementService {

    @Autowired
    private AdvertisementsMapper advertisementsMapper;
    @Autowired
    private OssProperties ossProperties;

    @Override
    public List<AdvertisementVo> list(QueryAdvertisementParam param) {
        List<AdvertisementVo> result = new ArrayList<>();
        String slug = param.getSlug();
        String isFetchOne = param.getIsFetchOne();
        String language = param.getLanguage();
        if(!StringUtils.isEmpty(language)) {
            language = CommonBusinessUtils.judgeLanguage(language);
        }
        AdvertisementsExample example = new AdvertisementsExample();
        AdvertisementsExample.Criteria criteria = example.createCriteria();
        if(!StringUtils.isEmpty(slug)) {
            criteria.andSlugEqualTo(slug);
        }
        if(!StringUtils.isEmpty(slug)) {
            criteria.andLanguageEqualTo(LanguageEnum.getCodeByName(language));
        }
        //获取访问域名
        String viewUrl = ossProperties.getViewUrl();
        List<Advertisements> advertisementList = advertisementsMapper.selectByExample(example);
        if(!CollectionUtils.isEmpty(advertisementList) && "1".equals(isFetchOne)){
            AdvertisementVo vo = new AdvertisementVo();
            Advertisements adv = advertisementList.get(0);
            BeanUtils.copyProperties(adv,vo);
            vo.setImageUrl(viewUrl + adv.getImage());
            setExtraInfo(vo,adv);
            increaseHit(adv);
            result.add(vo);
        }else{
            for (Advertisements adv : advertisementList) {
                AdvertisementVo vo = new AdvertisementVo();
                BeanUtils.copyProperties(adv,vo);
                vo.setImageUrl(viewUrl+adv.getImage());
                setExtraInfo(vo,adv);
                result.add(vo);
            }
        }
        return result;
    }

    /**
     * 设置广告额外信息
     * @param vo 返回广告vo
     * @param adv 数据库查询广告信息
     * 需要注意第二个=号切割的时候，有可能出现没有list【1】的值，需要进行处理
     */
    public void setExtraInfo(AdvertisementVo vo, Advertisements adv){
        Map<String, String> map = new HashMap<>();
        String extraInfo = adv.getExtraInfo();
        if(!StringUtils.isEmpty(extraInfo)){
            String[] array = extraInfo.split("&");
            for (int i = 0; i < array.length; i++) {
                String[] list = array[i].split("=");
                if(list.length == 2) {
                    map.put(list[0], list[1]);
                }else{
                    map.put(list[0], null);
                }
            }
        }
        vo.setExtraInfo(map);
    }

    /**
     * 增加广告点击量
     * @param adv adv
     */
    public void increaseHit(Advertisements adv){
        adv.setTotalHits(adv.getTotalHits()+1);
        adv.setUpdatedAt(new Date());
        advertisementsMapper.updateByPrimaryKey(adv);
    }

}
