package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.client.OssFileClient;
import cn.kdan.pdf.backend.core.mapper.AvatarsMapper;
import cn.kdan.pdf.backend.core.model.Avatars;
import cn.kdan.pdf.backend.core.model.AvatarsExample;
import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.pojo.app.MemberVo;
import cn.kdan.pdf.backend.core.pojo.app.UpTokenVo;
import cn.kdan.pdf.backend.core.properties.OssProperties;
import cn.kdan.pdf.backend.core.service.AvatarService;
import cn.kdan.pdf.backend.core.service.MemberService;
import cn.kdan.pdf.backend.core.utils.CommonBusinessUtils;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;
import utils.CommonUtils;

import javax.activation.MimetypesFileTypeMap;
import javax.annotation.Resource;
import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * @author sfq
 * @description
 */
@Slf4j
@Service
public class AvatarServiceImpl implements AvatarService {

    @Resource
    private AvatarsMapper avatarsMapper;
    @Autowired
    private OssFileClient ossFileClient;
    @Autowired
    private MemberService memberService;
    @Autowired
    private OssProperties ossProperties;

    // 图片文件类型
    public static String IMG_TYPE_PNG = "PNG";
    public static String IMG_TYPE_JPG = "JPG";
    public static String IMG_TYPE_JPEG = "JPEG";
    public static String IMG_TYPE_DMG = "BMP";
    public static String IMG_TYPE_GIF = "GIF";
    public static String IMG_TYPE_SVG = "SVG";


    @Override
    public List<Avatars> list() {
        AvatarsExample example = new AvatarsExample();
        example.createCriteria().andMemberIdIsNull();
        List<Avatars> avatars = avatarsMapper.selectByExample(example);
        String viewUrl = ossProperties.getViewUrl();
        for (Avatars avatar : avatars) {
            avatar.setAvatar(viewUrl + avatar.getAvatar());
        }
        return avatars;
    }

    @Override
    public String selectAvatarByMemberId(String memberId) {
        AvatarsExample example = new AvatarsExample();
        example.createCriteria().andMemberIdEqualTo(memberId);
        List<Avatars> avatars = avatarsMapper.selectByExample(example);
        if(CollectionUtils.isEmpty(avatars)){
            return "";
        }
        String viewUrl = ossProperties.getViewUrl();
        return viewUrl + avatars.get(0).getAvatar();
    }

    @Override
    public UpTokenVo picUpload(MultipartFile file) {
        UpTokenVo upTokenVo = new UpTokenVo();
        checkFile(file);
        Members currentUser = memberService.getCurrentUser();
        String memberId = currentUser.getId();

        String ossFileName = "";
        try {
            //上传文件并授权为可读写
            ossFileName = ossFileClient.upload(file,true);
            //设置读权限
//            ossFileClient.setAcl(ossFileName);
        } catch (IOException e) {
            log.error("上传头像失败：{}",e.getMessage());
        }
        //判断是否已存在用户头像数据，有就更新，没有就新增
        AvatarsExample example = new AvatarsExample();
        example.createCriteria().andMemberIdEqualTo(memberId);
        List<Avatars> list = avatarsMapper.selectByExample(example);
        Date now = new Date();
        if(!CollectionUtils.isEmpty(list)){
            Avatars avatars = list.get(0);
            // 将用户原来的头像数据先删除
            String oldUrl = avatars.getAvatar();
            ossFileClient.delete(oldUrl);
            avatars.setAvatar(ossFileName);
            avatars.setUpdatedAt(now);
            avatarsMapper.updateByPrimaryKey(avatars);
        }else {
            Avatars avatars = new Avatars();
            avatars.setAvatar(ossFileName);
            avatars.setId(CommonUtils.generateId());
            avatars.setMemberId(memberId);
            avatars.setAppScope(1);
            avatars.setNumber(0);
            avatars.setCreatedAt(now);
            avatars.setUpdatedAt(now);
            avatarsMapper.insert(avatars);
        }
        MemberVo member = new MemberVo();
        BeanUtils.copyProperties(currentUser,member);
        member.setPhone(currentUser.getPhone());
        member.setName(currentUser.getName());
        member.setStatus("register");
        member.setTimeZone(currentUser.getTimeZone());
        member.setEmail(currentUser.getEmail());
        member.setPhoneZone(currentUser.getPhoneZone());
        String viewUrl = ossProperties.getViewUrl();
        member.setAvatarUrl(viewUrl+ossFileName);
        upTokenVo.setMember(member);
        return upTokenVo;
    }

    /**
     * 检查文件类型大小等
     * @param file
     */
    private void checkFile(MultipartFile file) {
        long size = file.getSize();
        String suffix = file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".") + 1);
        if(!(IMG_TYPE_DMG.equals(suffix.toUpperCase()) ||
                IMG_TYPE_GIF.equals(suffix.toUpperCase()) ||
                IMG_TYPE_JPEG.equals(suffix.toUpperCase()) ||
                IMG_TYPE_JPG.equals(suffix.toUpperCase()) ||
                IMG_TYPE_PNG.equals(suffix.toUpperCase()) ||
                IMG_TYPE_SVG.equals(suffix.toUpperCase()))){
            throw new BackendRuntimeException("请上传图片类型的文件！");
        }
        boolean flag = CommonBusinessUtils.checkFileSize(size, 2, "M");
        if(!flag){
            throw new BackendRuntimeException("上传图片太大！");
        }
    }

}
