package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.enums.MissionFileStatusEnum;
import cn.kdan.pdf.backend.core.mapper.ConvertTypesMapper;
import cn.kdan.pdf.backend.core.mapper.MissionFilesMapper;
import cn.kdan.pdf.backend.core.mapper.MissionsMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.pojo.ConvertTypeVo;
import cn.kdan.pdf.backend.core.pojo.MissionFileVO;
import cn.kdan.pdf.backend.core.pojo.MissionVO;
import cn.kdan.pdf.backend.core.service.ConvertFileService;
import cn.kdan.pdf.backend.core.service.MemberService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ConvertFileServiceImpl implements ConvertFileService {

    @Autowired
    private ConvertTypesMapper convertTypesMapper;
    @Autowired
    private MissionFilesMapper missionFilesMapper;
    @Autowired
    private MemberService memberService;
    @Autowired
    private MissionsMapper missionsMapper;

    /**
     * 权限错误
     */
    final static String PERMISSION_ERROR = "permission_error";
    /**
     * 损坏文件
     */
    final static String DAMAGED_FILE = "damaged_file";
    /**
     * 目标类型错误
     */
    final static String DETERMINE_TYPE_ERROR = "determine_type_error";

    @Override
    public List<ConvertTypeVo> getConvertTypeList(String inputFileType, String outputFileType) {
        List<ConvertTypeVo> result = new ArrayList<>();
        ConvertTypesExample example = new ConvertTypesExample();
        ConvertTypesExample.Criteria criteria = example.createCriteria();
        if(!StringUtils.isEmpty(outputFileType)){
            criteria.andOutputTypeEqualTo(outputFileType);
        }
        criteria.andInputTypeEqualTo(inputFileType).andDeletedAtIsNull();
        List<ConvertTypes> convertTypes = convertTypesMapper.selectByExample(example);

        for (ConvertTypes type : convertTypes) {
            ConvertTypeVo vo = new ConvertTypeVo();
            BeanUtils.copyProperties(type, vo);
            result.add(vo);
        }
        return result;
    }

    @Override
    public List<ConvertTypes> getAllConvertType() {
        ConvertTypesExample example = new ConvertTypesExample();
        example.createCriteria().andDeletedAtIsNull();
        return convertTypesMapper.selectByExample(example);
    }

    /**
     * 获取所有转换类型的集合
     * @return
     */
    public Map<Integer,ConvertTypes> getConvertTypeMap() {
        Map<Integer,ConvertTypes> map = new HashMap<>();

        ConvertTypesExample example = new ConvertTypesExample();
        example.createCriteria().andDeletedAtIsNull();
        List<ConvertTypes> convertTypes = convertTypesMapper.selectByExample(example);
        for (ConvertTypes convertType : convertTypes) {
            map.put(convertType.getId(),convertType);
        }
        return map;
    }

    @Override
    public List<MissionVO> getFileStatus(String mid) {
        List<MissionVO> result = new ArrayList<>();
        MissionVO missionVo = new MissionVO();
        missionVo.setId(mid);

        //暂时存放一个mission
        MissionFilesExample example = new MissionFilesExample();
        example.createCriteria().andMissionIdEqualTo(mid);
        List<MissionFiles> missionFiles = missionFilesMapper.selectByExample(example);
        //获取所有转档类型集合
        Map<Integer, ConvertTypes> convertTypeMap = getConvertTypeMap();
        BigDecimal totalSize = BigDecimal.ZERO;
        List<MissionFileVO> list = new ArrayList<>();
        for (MissionFiles file : missionFiles) {
            totalSize = totalSize.add(file.getSize());
            MissionFileVO vo = new MissionFileVO();
            BeanUtils.copyProperties(file, vo);
            vo.setSize(file.getSize().setScale(3, BigDecimal.ROUND_HALF_UP));
            vo.setCreateTime(file.getCreatedAt());
            vo.setNewPrice(file.getPrice());
            //设置retryEnable
            boolean retryEnabled = true;
            if (!StringUtils.isEmpty(file.getFailReason())) {
                String s = file.getFailReason().split("@@")[0];
                retryEnabled = !s.contains(PERMISSION_ERROR) && !s.contains(DAMAGED_FILE) && !s.contains(DETERMINE_TYPE_ERROR);
            }
            vo.setRetryEnabled(retryEnabled);
            ConvertTypes convertType = convertTypeMap.get(file.getConvertTypeId());
            vo.setSourceType(convertType.getInputType());
            vo.setSingleMid(file.getId());
            vo.setTargetType(convertType.getOutputType());
            // 设置转换时间
            if (file.getConvertEndAt() != null && file.getConvertAt() != null) {
                long l = file.getConvertEndAt().getTime() - file.getConvertAt().getTime();
                vo.setTimeTakenForConvert((int) l);
            }else {
                vo.setTimeTakenForConvert(0);
            }
            vo.setStatusName(MissionFileStatusEnum.getNameByValue(vo.getStatus()));
            list.add(vo);
        }
        missionVo.setMissionFilePoJos(list);
        Missions missions = missionsMapper.selectByPrimaryKey(mid);
        missionVo.setTotalPrice(missions.getPrice());
        missionVo.setTargetType(list.get(0).getTargetType());
        totalSize = totalSize.setScale(3, BigDecimal.ROUND_HALF_UP);
        missionVo.setTotalSize(totalSize);
        result.add(missionVo);
        return result;
    }

    /**
     * 计算价格
     *
     * @return 券数
     */
    public int calculatePrice(ConvertTypes convertType) {
        int price;
        Members member = memberService.getCurrentUser();

        //todo env函数处理
        if (member.getSubscriberType() == 1) {
            price = convertType.getSpPrice();
        } else if (convertType == null) {
            price = 1;
        } else {
            price = convertType.getPrice();
        }
        return price;
    }
}
