package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.constant.AuthConstant;
import cn.kdan.pdf.backend.core.model.OauthClientDetails;
import cn.kdan.pdf.backend.core.service.AuthService;
import constant.CommonConstant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.oauth2.provider.*;
import org.springframework.security.oauth2.provider.client.BaseClientDetails;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * @author tangxiangan
 */
public class CustomClientDetailsServiceImpl implements ClientDetailsService, ClientRegistrationService {

    private Logger logger = LoggerFactory.getLogger(CustomClientDetailsServiceImpl.class);

    private AuthService authService;

    public CustomClientDetailsServiceImpl(AuthService authService) {
        this.authService = authService;
    }

    @Override
    public ClientDetails loadClientByClientId(String clientId) throws ClientRegistrationException {
        OauthClientDetails clientDetailsVO = authService.getById(clientId);
        return convert(clientDetailsVO);
    }

    @Override
    public void addClientDetails(ClientDetails clientDetails) throws ClientAlreadyExistsException {
        logger.info("addClientDetails");
    }

    @Override
    public void updateClientDetails(ClientDetails clientDetails) throws NoSuchClientException {
        logger.info("updateClientDetails");
    }

    @Override
    public void updateClientSecret(String clientId, String secret) throws NoSuchClientException {
        logger.info("updateClientSecret");
    }

    @Override
    public void removeClientDetails(String clientId) throws NoSuchClientException {
        logger.info("removeClientDetails");
    }

    @Override
    public List<ClientDetails> listClientDetails() {
        List<OauthClientDetails> list = authService.listAll();
        List<ClientDetails> res = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            list.forEach(details -> res.add(convert(details)));
        }
        return res;
    }

    private BaseClientDetails convert(OauthClientDetails client) {
        BaseClientDetails clientDetails = new BaseClientDetails();
        clientDetails.setClientId(client.getClientId());
        clientDetails.setClientSecret(client.getClientSecret());
        clientDetails.setAccessTokenValiditySeconds(client.getAccessTokenValidity());
        clientDetails.setRefreshTokenValiditySeconds(client.getRefreshTokenValidity());
        if (!StringUtils.isEmpty(client.getScope())) {
            clientDetails.setScope(Arrays.asList(client.getScope().split(CommonConstant.STRING_SIGN_COMMA)));
        } else {
            clientDetails.setScope(Collections.emptySet());
        }
        if (!StringUtils.isEmpty(client.getAuthorizedGrantTypes())) {
            clientDetails.setAuthorizedGrantTypes(Arrays.asList(client.getAuthorizedGrantTypes().split(CommonConstant.STRING_SIGN_COMMA)));
        } else {
            clientDetails.setAuthorizedGrantTypes(Collections.emptyList());
        }

        if (!StringUtils.isEmpty(client.getWebServerRedirectUri())) {
            clientDetails.setRegisteredRedirectUri(new LinkedHashSet<>(Collections.singletonList(AuthConstant.DEFAULT_REDIRECT_URI)));
        } else {
            clientDetails.setRegisteredRedirectUri(Collections.emptySet());
        }

        if (!StringUtils.isEmpty(client.getAutoapprove())) {
            String[] strAutoapproves = client.getAutoapprove().split(CommonConstant.STRING_SIGN_COMMA);
            HashSet<String> autoapprove = new HashSet<>(Arrays.asList(strAutoapproves));
            clientDetails.setAutoApproveScopes(autoapprove);
        } else {
            clientDetails.setAutoApproveScopes(Collections.emptySet());
        }

        //暂时不用
        //clientDetails.setAuthorities(Collections.emptyList());
        // clientDetails.setResourceIds(Collections.emptySet());
        // clientDetails.setAdditionalInformation(Collections.emptyMap());
        return clientDetails;
    }

}
