package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.constant.AuthConstant;
import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import pojo.CustomUserDetails;
import pojo.ResultMap;
import utils.JsonUtils;

import java.util.HashSet;

/**
 * @author tangxiangan
 */
@Service
public class CustomUserDetailsServiceImpl implements UserDetailsService {

    @Autowired
    private UserService userService;

    /**
     * 用于认证时如果从内存中找不到用户信息，从数据库查找用户信息
     *
     * @param account
     * @return UserDetails
     * @throws UsernameNotFoundException
     */
    @Override
    public UserDetails loadUserByUsername(String account) throws UsernameNotFoundException {
        Members user = userService.getByAccount(account);
        if (ObjectUtils.isEmpty(user)) {
            throw new UsernameNotFoundException(JsonUtils.getJsonString(new ResultMap<>(AuthConstant.EXCEPTION_CODE_USER_NOT_FOUND, AuthConstant.EXCEPTION_MSG_USER_NOT_FOUND)));
        }
        // 邮箱登陆时判断邮箱是否验证 没验证不能登陆
//        if (user.getEmail().equals(account) && user.getEmailStatus().equals(EmailStatusEnum.INACTIVE.value())) {
//            throw new UsernameNotFoundException(JsonUtils.getJsonString(new ResultMap<>(AuthConstant.EXCEPTION_CODE_EMAIL_LOGIN_FAIL, AuthConstant.EXCEPTION_MSG_EMAIL_NOT_VERIFY)));
//        }

        return new CustomUserDetails(user.getId(), user.getName(), user.getPasswordDigest(), true, true, true, true, new HashSet<>());
    }
}
