package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.mapper.DevicesMapper;
import cn.kdan.pdf.backend.core.model.Devices;
import cn.kdan.pdf.backend.core.model.DevicesExample;
import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.params.RelationParam;
import cn.kdan.pdf.backend.core.service.DeviceService;
import cn.kdan.pdf.backend.core.service.MemberService;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import pojo.Device;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author SongFuQiang
 * @description
 */
@Slf4j
@Service
public class DeviceServiceImpl implements DeviceService {

    @Resource
    private DevicesMapper devicesMapper;
    @Resource
    private MemberService memberService;

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public int insert(Devices devices) {
        Date now = new Date();
        devices.setCreatedAt(now);
        devices.setUpdatedAt(now);
        return devicesMapper.insert(devices);
    }

    @Override
    public Devices selectByMemberId(String memberId, String uuid) {
        //todo 最多允许三台设备
        DevicesExample example = new DevicesExample();
        DevicesExample.Criteria criteria = example.createCriteria();
        criteria.andMemberIdEqualTo(memberId);
        example.setOrderByClause("updated_at ASC");
        //先根据用户id查询设备信息
        List<Devices> devices = devicesMapper.selectByExample(example);
        if(CollectionUtils.isEmpty(devices)){
            return null;
        }
        criteria.andUuidEqualTo(uuid);
        //再根据用户id+设备唯一标识uuid查询
        List<Devices> list = devicesMapper.selectByExample(example);
        //如果数据大于等于3条，不允许新增数据，只能更新
        if(!CollectionUtils.isEmpty(list)){
            return list.get(0);
        }else{
            if(devices.size() >= 3){
                throw new BackendRuntimeException("当前已登录3台设备，无法在进行登录");
            }else{
                return null;
            }
        }
//        if(devices.size() >= 3 && CollectionUtils.isEmpty(list)){
//            return devices.get(0);
//        }else if ((devices.size() >= 3 && !CollectionUtils.isEmpty(list))){
//            return list.get(0);
//        }else {
//            return null;
//        }
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public void update(Devices device, String uuid) {
        Date date = new Date();
        device.setUpdatedAt(date);
        devicesMapper.updateByPrimaryKey(device);
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public Device updateInfo(RelationParam param) {
        Members currentUser = memberService.getCurrentUser();
        Device device = param.getDevice();
        DevicesExample example = new DevicesExample();
        example.createCriteria().andMemberIdEqualTo(currentUser.getId()).andUuidEqualTo(device.getUuid());
        List<Devices> devices = devicesMapper.selectByExample(example);
        if(!CollectionUtils.isEmpty(devices)){
            Devices dev = devices.get(0);
            BeanUtils.copyProperties(device,dev);
            dev.setUpdatedAt(new Date());
            devicesMapper.updateByExample(dev,example);
        }
        return device;
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public void deleteByUuid(String uuid, String memberId){
        DevicesExample example = new DevicesExample();
        example.createCriteria().andMemberIdEqualTo(memberId).andUuidEqualTo(uuid);
        devicesMapper.deleteByExample(example);
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public void clearDevice(String memberId) {
        DevicesExample example = new DevicesExample();
        example.createCriteria().andMemberIdEqualTo(memberId);
        devicesMapper.deleteByExample(example);
    }
}
