package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.mapper.LocationsMapper;
import cn.kdan.pdf.backend.core.model.Locations;
import cn.kdan.pdf.backend.core.model.LocationsExample;
import cn.kdan.pdf.backend.core.service.LocationService;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author sfq
 * @description
 */
@Slf4j
@Service
public class LocationServiceImpl implements LocationService {

    @Resource
    private LocationsMapper locationsMapper;

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public int insert(Locations locations) {
        Date now = new Date();
        locations.setCreatedAt(now);
        locations.setUpdatedAt(now);
        return locationsMapper.insert(locations);
    }

    @Override
    public Locations selectByMemberId(String memberId) {
        LocationsExample example = new LocationsExample();
        example.createCriteria().andMemberIdEqualTo(memberId);
        List<Locations> locations = locationsMapper.selectByExample(example);
        if(CollectionUtils.isEmpty(locations)){
            return null;
        }
        return locations.get(0);
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public void update(Locations location) {
        Date date = new Date();
        location.setUpdatedAt(date);
        LocationsExample example = new LocationsExample();
        example.createCriteria().andMemberIdEqualTo(location.getMemberId());
        locationsMapper.updateByExampleSelective(location, example);
    }
}
