package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.client.ComPdfKitClient;
import cn.kdan.pdf.backend.core.constant.MissionConstant;
import cn.kdan.pdf.backend.core.constant.MissionFilesConstant;
import cn.kdan.pdf.backend.core.enums.MissionFileStatusEnum;
import cn.kdan.pdf.backend.core.mapper.ext.ExtMissionFilesMapper;
import cn.kdan.pdf.backend.core.model.MissionFiles;
import cn.kdan.pdf.backend.core.model.MissionFilesExample;
import cn.kdan.pdf.backend.core.pojo.MissionFileVO;
import cn.kdan.pdf.backend.core.pojo.comPdfKit.UploadFileResult;
import cn.kdan.pdf.backend.core.service.MissionFilesService;
import cn.kdan.pdf.backend.core.utils.ComPdfKitUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;
import utils.FileUtils;

import javax.annotation.Resource;
import java.io.File;
import java.util.Date;
import java.util.List;

/**
 * @author tangxiangan
 */
@Service
@Slf4j
public class MissionFileServiceImpl implements MissionFilesService {
    @Autowired
    private ExtMissionFilesMapper missionFilesMapper;
    @Resource
    ComPdfKitUtils comPdfKitUtils;
    @Override
    public PageInfo<MissionFileVO> page(Integer page, Integer pageSize, String memberId) {
        PageHelper.startPage(page, pageSize);
        List<MissionFileVO> missionFilesList = missionFilesMapper.getMissionFileListWithConvertType(memberId);
        missionFilesList.forEach(item->{
            item.setStatusName(MissionFileStatusEnum.getNameByValue(item.getStatus()));
        });
        return new PageInfo<>(missionFilesList);
    }

    @Override
    public Boolean delete(List<String> ids) {
        MissionFilesExample example = new MissionFilesExample();
        example.createCriteria().andIdIn(ids);
        MissionFiles missionFile = new MissionFiles();
        missionFile.setDeletedAt(new Date());
        missionFilesMapper.updateByExampleSelective(missionFile, example);
        return Boolean.TRUE;
    }

    @Override
    public void insert(MissionFiles missionFile) {
        missionFilesMapper.insert(missionFile);
    }

    @Override
    public Boolean upload(MultipartFile file, String missionFileId) {
            MissionFiles missionFile = getById(missionFileId);
        try {
            formatChecks(file);
            File convertFile = FileUtils.multipartFileToFile(file);
            ComPdfKitClient client = comPdfKitUtils.getComPdfKitClient();
            UploadFileResult result  = client.uploadFile(convertFile, missionFile.getSaasTaskId(), "");
            missionFile.setProcessId(result.getFileKey());
            missionFile.setTraceResult(result.getFileUrl());
            missionFilesMapper.updateByPrimaryKeySelective(missionFile);
        } catch (Exception e) {
            log.error("上传文件失败，{}，{}", e, e.getMessage());
            missionFile.setStatus(MissionFileStatusEnum.FAILED.value());
            missionFile.setFailReason(e.getMessage());
            missionFilesMapper.updateByPrimaryKeySelective(missionFile);
            throw new BackendRuntimeException(e.getMessage());
        }
        return Boolean.TRUE;
    }

    public void formatChecks(MultipartFile file) {
        String fileName = file.getOriginalFilename();
        log.info("文件名:{}",fileName);
        if (fileName == null || fileName.indexOf('.') == -1 || fileName.indexOf('.') >= fileName.length() - 1) {
            throw new BackendRuntimeException(MissionFilesConstant.ERROR_FILE_FORMAT);
        }
        if (file.getSize() <= 0) {
            // 文件大小为0异常
            throw new BackendRuntimeException(MissionFilesConstant.FILE_SIZE_ZONE_ERROR);
        }
    }

    @Override
    public MissionFiles getById(String missionFileId) {
        MissionFiles missionFile = missionFilesMapper.selectByPrimaryKey(missionFileId);
        if(ObjectUtils.isEmpty(missionFile)){
            throw new BackendRuntimeException(MissionFilesConstant.EXCEPTION_MSG_FILE_NOT_EXIST);
        }
        return missionFile;
    }

    @Override
    public void update(MissionFiles missionFiles) {
        missionFilesMapper.updateByPrimaryKeySelective(missionFiles);
    }

    @Override
    public MissionFiles queryByProcessId(String processId) {
        MissionFilesExample example = new MissionFilesExample();
        example.createCriteria().andProcessIdEqualTo(processId);
        List<MissionFiles> missionFiles = missionFilesMapper.selectByExample(example);
        MissionFiles file = null;
        if(!CollectionUtils.isEmpty(missionFiles)){
            file = missionFiles.get(0);
        }
        return file;
    }

    @Override
    public void updateBySaasTaskId(String taskId) {
        missionFilesMapper.updateBySaasTaskId(taskId,new Date());
    }

    @Override
    public void updateByProcessId(String processId, MissionFileStatusEnum status) {
        missionFilesMapper.updateByProcessId(processId, status.value());
    }

    @Override
    public List<MissionFiles> getBySaasId(String id) {
        MissionFilesExample example = new MissionFilesExample();
        example.createCriteria().andSaasTaskIdEqualTo(id);
        return missionFilesMapper.selectByExample(example);
    }

    @Override
    public Integer getTotalConvertFile() {
        MissionFilesExample example = new MissionFilesExample();
        long l = missionFilesMapper.countByExample(example);
        return (int)l;
    }

}
