package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.constant.SubscriptionConstant;
import cn.kdan.pdf.backend.core.enums.*;
import cn.kdan.pdf.backend.core.mapper.ExpensesMapper;
import cn.kdan.pdf.backend.core.mapper.PricingDiscountsMapper;
import cn.kdan.pdf.backend.core.mapper.SubscriptionsMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.params.ExpenseParam;
import cn.kdan.pdf.backend.core.pojo.SubscriptionVO;
import cn.kdan.pdf.backend.core.pojo.app.*;
import cn.kdan.pdf.backend.core.service.*;
import cn.kdan.pdf.backend.core.utils.PeriodUtils;
import cn.kdan.pdf.backend.core.utils.wxpay.MyConfig;
import cn.kdan.pdf.backend.core.utils.wxpay.WXPayConstants;
import cn.kdan.pdf.backend.core.utils.wxpay.WXPayUtil;
import com.alibaba.fastjson.JSON;
import enums.CommonEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import utils.CommonUtils;
import utils.DateUtils;

import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;


/**
 * @author tangxiangan
 */
@Slf4j
@Service
public class SubscriptionServiceImpl implements SubscriptionService {

    @Autowired
    private SubscriptionsMapper subscriptionsMapper;
    @Autowired
    private SetPricingService setPricingService;
    @Autowired
    private PricingService pricingService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private OrderService orderService;
    @Autowired
    private PricingDiscountsMapper pricingDiscountsMapper;
    @Autowired
    private ExpensesMapper expensesMapper;
    @Autowired
    private MyConfig myWXConfig;

    @Override
    public long countTimesById(String memberId, String discountId) {
        SubscriptionsExample example = new SubscriptionsExample();
        example.createCriteria().andStatusNotEqualTo(SubscriptionStatusEnum.UNPAYED.value()).andPricingDiscountIdEqualTo(discountId).andMemberIdEqualTo(memberId);
        return subscriptionsMapper.countByExample(example);
    }

    @Override
    public CreateSubVo appCreate(String client, Integer payment, String targetType, String targetId) {
        CreateSubVo createSubVo = new CreateSubVo();
        //业务流程开始
        SubscriptionVO subscriptionVO = create(client, payment, targetType, targetId);
        BeanUtils.copyProperties(subscriptionVO,createSubVo);
        ThirdOrderVo orderVo = new ThirdOrderVo();
        Orders order = subscriptionVO.getOrder();
        BeanUtils.copyProperties(order, orderVo);
        createSubVo.setOrder(orderVo);
        // 设置androidSign
        if(PaymentEnum.ALIPAY.value().equals(payment)){
            orderVo.setAndroidSign(order.getAndroidSign());
        }else{
            WxAndroidSignVo wxAndroidSign = new WxAndroidSignVo();

            String androidSign = order.getAndroidSign();
            Map map = JSON.parseObject(androidSign, Map.class);
            try {
                // 调起支付接口签名返回给app端,和统一下单参数的签名不一样，因为参数不一样
                Map<String, String> signMmap = new HashMap<>();
                signMmap.put("appid", myWXConfig.getAppID());
                signMmap.put("partnerid", myWXConfig.getMchID());
                signMmap.put("prepayid", map.get(WXPayConstants.PREPAY_ID).toString());
                signMmap.put("package", "Sign=WXPay");
                signMmap.put("noncestr", WXPayUtil.generateNonceStr());
                signMmap.put("timestamp", String.valueOf(System.currentTimeMillis()).substring(0, 10));
                String sign = WXPayUtil.generateSignature(signMmap, myWXConfig.getKey(), WXPayConstants.SignType.MD5);

                wxAndroidSign.setNonceStr(signMmap.get("noncestr"));
                wxAndroidSign.setPrepayId(signMmap.get("prepayid"));
                wxAndroidSign.setAppId(signMmap.get("appid"));
                wxAndroidSign.setPartnerId(signMmap.get("partnerid"));
                wxAndroidSign.setTimestamp(signMmap.get("timestamp"));
                wxAndroidSign.setPackageSign(signMmap.get("package"));
                wxAndroidSign.setSign(sign);
                orderVo.setWxAndroidSign(wxAndroidSign);
            }catch (Exception e){
                log.info("签名出错"+e.getMessage());
            }
        }

        if(GoodsTypeEnum.SetPricing.value().equals(targetType)){
            List<SetPricings> list = setPricingService.getForSaleRMBById(targetId);
            if(!CollectionUtils.isEmpty(list)){
                createSubVo.setSetPricing(list.get(0));
            }
        }else if(GoodsTypeEnum.Pricing.value().equals(targetType)){
            List<Pricings> list = pricingService.getForSaleRMBById(targetId);
            if(!CollectionUtils.isEmpty(list)){
                createSubVo.setPricing(list.get(0));
            }
        }

        return createSubVo;
    }

    @Transactional(rollbackFor = BackendRuntimeException.class)
    @Override
    public SubscriptionVO create(String client, Integer payment, String targetType, String targetId) {
        //判断平台类型
        client = StringUtils.isEmpty(client) ? ClientEnum.APP.value() : client;
        if(!PaymentEnum.checkEnumByValue(payment)){
            throw new BackendRuntimeException(SubscriptionConstant.EXCEPTION_MSG_INVALID_PAYMENT);
        }
        if (GoodsTypeEnum.SetPricing.value().equals(targetType)) {
            List<SetPricings> list = setPricingService.getForSaleRMBById(targetId);
            return getSubscriptionPojo(client, payment, targetType, targetId, CollectionUtils.isEmpty(list));
        }else if(GoodsTypeEnum.Pricing.value().equals(targetType)){
            List<Pricings> list = pricingService.getForSaleRMBById(targetId);
            return getSubscriptionPojo(client, payment, targetType, targetId, CollectionUtils.isEmpty(list));
        }
        return null;
    }

    /**
     *
     * 查找用户没有没未支付的指定类型的订阅 如果没有则生成
     */
    public Subscriptions findMemberUnPayedSubscription(Members member, String targetType, Integer targetId,Integer payment,String client) {
        SubscriptionsExample example = new SubscriptionsExample();
        example.createCriteria().andMemberIdEqualTo(member.getId()).andTargetIdEqualTo(targetId).andPaymentEqualTo(payment).andTargetTypeEqualTo(targetType).andStatusEqualTo(SubscriptionStatusEnum.UNPAYED.value());
        List<Subscriptions> subscriptionsList = subscriptionsMapper.selectByExample(example);
        //如果该用户没有未支付的订阅记录,则插入一条
        if (GoodsTypeEnum.SetPricing.value().equals(targetType)) {
            SetPricings setPricings = setPricingService.getById(targetId);
            return getSubscriptions(member, targetType, targetId, payment, client, subscriptionsList, setPricings);
        }else if(GoodsTypeEnum.Pricing.value().equals(targetType)) {
            Pricings pricings = pricingService.getById(targetId);
            return getSubscriptions(member, targetType, targetId, payment, client, subscriptionsList, pricings);
        }
        return null;
    }


    private Subscriptions getSubscriptions(Members member, String targetType, Integer targetId, Integer payment, String client, List<Subscriptions> subscriptionsList, Object target) {
        if (CollectionUtils.isEmpty(subscriptionsList)) {
            Subscriptions subscriptions = getSubscriptions(member, targetType, targetId, payment);
            // 计算订阅的开始时间和结束时间
            renewInfo(true, client, subscriptions, member, target);
            subscriptionsMapper.insert(subscriptions);
            return subscriptions;
        } else {
            //即使已经存在订阅要重新计算订阅开始时间
            Date now = new Date();
            subscriptionsList.get(0).setUpdatedAt(now);
            subscriptionsList.get(0).setCreatedAt(now);
            renewInfo(true, client, subscriptionsList.get(0), member, target);
            subscriptionsMapper.updateByPrimaryKeySelective(subscriptionsList.get(0));
            return subscriptionsList.get(0);
        }
    }

    /**
     * 获取订阅pojo对象
     * @param client 客户端（web、android）
     * @param payment 支付方式（0：支付宝、1：微信）
     * @param targetType 购买服务（SetPricing、Pricing）
     * @param targetId 服务id
     * @param empty 查询购买的服务是否为空
     * @return 订阅pojo
     */
    private SubscriptionVO getSubscriptionPojo(String client, Integer payment, String targetType, String targetId, boolean empty) {
        if (empty) {
            throw new BackendRuntimeException(SubscriptionConstant.EXCEPTION_MSG_INVALID_TARGET_ID);
        }
        Members members = memberService.getCurrentUser();
        Integer subscribeType = memberService.judgeSubscribeType(members.getId());
        members.setSubscriberType(subscribeType);
        //查找用户未支付状态的会员服务 没有则插入
        Subscriptions subscription = findMemberUnPayedSubscription(members, targetType, Integer.parseInt(targetId), payment, client);
        //查找用户有没有没有支付的微信订单，没有则插入(填充用户订单信息)
        Orders order = orderService.findMemberUnPayedOrders(members.getId(), subscription, client);
        SubscriptionVO pojo = new SubscriptionVO();
        BeanUtils.copyProperties(subscription, pojo);
        pojo.setOrder(order);
        return pojo;
    }

    /**
     * 对订阅对象赋值
     * @param member 用户
     * @param targetType 购买服务（SetPricing、Pricing）
     * @param targetId 服务id
     * @param payment 支付方式（0：支付宝、1：微信）
     * @return 订阅对象
     */
    private Subscriptions getSubscriptions(Members member, String targetType, Integer targetId, Integer payment) {
        Subscriptions subscriptions = new Subscriptions();
        subscriptions.setId(CommonUtils.generateId());
        subscriptions.setMemberId(member.getId());
        subscriptions.setTargetType(targetType);
        subscriptions.setTargetId(targetId);
        subscriptions.setStatus(SubscriptionStatusEnum.UNPAYED.value());
        subscriptions.setCreatedAt(new Date());
        subscriptions.setUpdatedAt(new Date());
        subscriptions.setPayment(payment);
        return subscriptions;
    }

    @Override
    public void renewInfo(Boolean force, String client, Subscriptions subscription, Members member, Object target) {
        client = StringUtils.isEmpty(client) ? "web" : client;
        //填充价格，总券数总时间
        discountInfo(subscription,client,member,target,subscription.getTargetType());
        if(StringUtils.isEmpty(subscription.getTargetId())){
            subscription.setStartDate(new Date());
            //开始时间加上结束时间
            subscription.setEndDate(!StringUtils.isEmpty(subscription.getTotalPeriod())? PeriodUtils.countDate(subscription.getStartDate(),subscription.getTotalPeriod()) : subscription.getStartDate());
        }else {

            Subscriptions lastSubs = getLastSub(subscription,member,target);
            //当前订阅不等于上一个订阅且上一个订阅的结束日期不为空时，设置当前订阅的开始日期为上一个订阅的结束日期加1秒；否则设置开始日期为当前时间
            if(!ObjectUtils.isEmpty(lastSubs)&&!subscription.getId().equals(lastSubs.getId()) && lastSubs.getEndDate() != null){
                LocalDateTime localDateTime = PeriodUtils.covertLocalDateTime(lastSubs.getEndDate());
                localDateTime.plus(1L, ChronoUnit.SECONDS);
                subscription.setStartDate(PeriodUtils.covertDate(localDateTime));
            } else{
                subscription.setStartDate(new Date());
            }

            /*
                1.调用countPeriod方法有无值、有则继续
                  sub中referral_id是否为空,不为空设置period为7.days
                  直接设置period为period或者countPeriod
                2.计算t_period值
                3.如果t_period大于等于1.days
                  endDate=（startDate+t_period）的一天开始减一秒（23.59.59）
                  否则endDate=startDate+t_period
             */
            PricingDiscounts pricingDiscounts = pricingDiscountsMapper.selectByPrimaryKey(subscription.getPricingDiscountId());

            String period;
            //进行period的赋值
            if(subscription.getTargetType().equals(CommonEnum.SubscriptionTypeEnum.SET_PRICING.value())){
                //获取setPricing对象
                SetPricings setPricing = (SetPricings) target;
                period = setPricingService.countPeriod(setPricing, pricingDiscounts);
            } else{
                //获取pricing对象
                Pricings pricing = (Pricings) target;
                period = pricingService.countPeriod(pricing, pricingDiscounts);
            }
            if(!StringUtils.isEmpty(period)){
                //进行period的赋值
                if(!StringUtils.isEmpty(subscription.getMemberReferralId())){
                    subscription.setTotalPeriod("7.days");
                } else{
                    subscription.setTotalPeriod(period);
                }
            }
            //设置endDate
            if(!StringUtils.isEmpty(period)) {
                List<String> periodList = Arrays.stream(period.split("\\.")).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(periodList)) {
                    String num = periodList.get(0);
                    String unit = periodList.get(1);
                    if ("days".equals(unit) && 1 <= Integer.parseInt(num)) {
                        // 取结束日期前一天的结束时间（23:59:59）
                        Date endDate = PeriodUtils.countDate(subscription.getStartDate(), period);
                        LocalDateTime localDateTime = PeriodUtils.covertLocalDateTime(endDate);
                        Date date = PeriodUtils.covertDate(localDateTime.minusSeconds(1));
                        subscription.setEndDate(date);
                    } else {
                        subscription.setEndDate(PeriodUtils.countDate(subscription.getStartDate(), period));
                    }
                }
            }
        }
    }

    /**
     * 判断是否有活跃折扣，设置订阅的折扣id，价格和总券数，时间
     * @param subscriptions 订阅
     * @param client 客户端
     * @param member 用户
     * @param target setpricing pricing对象
     * @param targetType  类型
     */
    private void discountInfo (Subscriptions subscriptions,String client,Members member,Object target, String targetType) {
        if (GoodsTypeEnum.SetPricing.value().equals(targetType)){
            SetPricings setPricing = (SetPricings)target;
            //如果有折扣id的话查询折扣id，如果没有折扣id则返回null
            PricingDiscounts activeDiscount = !StringUtils.isEmpty(setPricing.getPricingDiscountId()) ? setPricingService.getActiveDiscount(setPricing.getPricingDiscountId(),member): null;
            //如果有活跃折扣设置活跃折扣id
            subscriptions.setPricingDiscountId(!ObjectUtils.isEmpty(activeDiscount)? activeDiscount.getId() : null);
            subscriptions.setPrice(setPricingService.countPrice(setPricing,activeDiscount,member,client));
            subscriptions.setTotalPoints(setPricingService.countPointsAmount(setPricing,activeDiscount));
            subscriptions.setTotalPeriod(setPricingService.countPeriod(setPricing,activeDiscount));
        }else{
            // pricing
            Pricings pricing = (Pricings)target;
            //如果有折扣id的话查询折扣id，如果没有折扣id则返回null
            PricingDiscounts activeDiscount = !StringUtils.isEmpty(pricing.getPricingDiscountId()) ? setPricingService.getActiveDiscount(pricing.getPricingDiscountId(),member): null;
            //如果有活跃折扣设置活跃折扣id
            subscriptions.setPricingDiscountId(!ObjectUtils.isEmpty(activeDiscount)? activeDiscount.getId() : null);
            subscriptions.setPrice(pricingService.countPrice(pricing,activeDiscount,member,client));
            subscriptions.setTotalPoints(pricingService.countPointsAmount(pricing,activeDiscount));
            subscriptions.setTotalPeriod(pricingService.countPeriod(pricing,activeDiscount));
        }


    }

    /**
     * 获取上一个订阅记录
     * @param subscriptions 订阅
     * @param members 用户
     * @param obj setPricing或pricing
     * @return 上一个订阅记录
     */
    public Subscriptions getLastSub(Subscriptions subscriptions, Members members, Object obj){
        Integer targetId = subscriptions.getTargetId(); //会员服务id或券服务id
        if(ObjectUtils.isEmpty(members)){
            return subscriptions;
        }
        if(StringUtils.isEmpty(targetId)){
            return null;
        }
        Subscriptions lastSub = null;
        if(CommonEnum.SubscriptionTypeEnum.SET_PRICING.value().equals(subscriptions.getTargetType())){
            SetPricings setPricings = setPricingService.getById(targetId);
            if(SetPricingModeEnum.member_v2.value().equals(setPricings.getMode())){
                // 拿到结束时间最晚的一个订阅作为上一个订阅
                List<Subscriptions> list = getSubscriptionList(subscriptions,members);
                for (Subscriptions subs : list) {
                    if(lastSub != null){
                        return lastSub;
                    } else if(!ObjectUtils.isEmpty(subs) && !setPricings.getMode().equals(SetPricingModeEnum.free.value())){
                        lastSub = subs;
                    }
                }
            }
        } else{
            SubscriptionsExample example = new SubscriptionsExample();
            example.createCriteria().andMemberIdEqualTo(members.getId()).andStatusEqualTo(SubscriptionStatusEnum.ACTIVED.value())
                    .andIdNotEqualTo(subscriptions.getId()).andTargetIdEqualTo(subscriptions.getTargetId()).andTargetTypeEqualTo(GoodsTypeEnum.Pricing.value());
            List<Subscriptions> list = subscriptionsMapper.selectByExample(example);
            if(!CollectionUtils.isEmpty(list)){
                lastSub = list.get(0);
            }
        }
        return lastSub;
    }

    /**
     * 获取订阅记录集合
     * @param sub 订阅
     * @param members 用户
     * @return 订阅记录集合
     */
    public List<Subscriptions> getSubscriptionList(Subscriptions sub, Members members){
        SubscriptionsExample example = new SubscriptionsExample();
        example.createCriteria().andMemberIdEqualTo(members.getId())
                .andStatusEqualTo(SubscriptionStatusEnum.ACTIVED.value()).andTargetTypeEqualTo(sub.getTargetType())
                .andIdNotEqualTo(sub.getId()).andEndDateGreaterThanOrEqualTo(new Date()).andTargetIdNotEqualTo(1);
        example.setOrderByClause("end_date DESC");
        return subscriptionsMapper.selectByExample(example);
    }

    @Override
    public void update(Subscriptions subscription) {
        subscriptionsMapper.updateByPrimaryKeySelective(subscription);
    }

    @Override
    public Subscriptions getById(String subscriptionId) {
        return subscriptionsMapper.selectByPrimaryKey(subscriptionId);
    }

    @Override
    public RemindVo remindRenew() {
        Members currentUser = memberService.getCurrentUser();
        String memberId = currentUser.getId();

        //查询出mode为0的setPricing且生效的订阅
        List<Subscriptions> list = setPricingService.selectMode0(memberId);
        //查出即将过期的订阅
        Date now = new Date();
        Date date1 = DateUtils.addDate(now, -7);
        SubscriptionsExample example = new SubscriptionsExample();
        example.setOrderByClause("start_date desc");
        SubscriptionsExample.Criteria criteria = example.createCriteria();
        List<Integer> statusList = new ArrayList<>();
        statusList.add(2);
        statusList.add(3);
        criteria.andTargetTypeEqualTo("SetPricing").andMemberIdEqualTo(memberId).andEndDateBetween(date1,now).andStatusIn(statusList);
        List<Subscriptions> subscriptions = subscriptionsMapper.selectByExample(example);
        RemindVo vo = new RemindVo();
        if(list.size() == 0 && !CollectionUtils.isEmpty(subscriptions)){
            Subscriptions expire = subscriptions.get(0);
            vo.setRemindType("set_pricing");
            int expireDay = (int) (expire.getEndDate().getTime() - now.getTime()) / 86400000;
            String remindMsg = "你的会员还有"+expireDay+"天到期，续费享受享7折优惠哦";
            vo.setRemindMsg(remindMsg);
            vo.setFreeSubscription(null);
        }else{
            List<Subscriptions> list1 = setPricingService.selectPByModeAndMemberId(memberId, "1");
            if(!CollectionUtils.isEmpty(list1) && list1.size() >= 1){
                vo.setRemindType("space");
            }
        }
        return vo;
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public ExpenseVo expense(ExpenseParam param) {
        Members currentUser = memberService.getCurrentUser();
        Integer points = param.getPoints();
        //参数校验
        if(points == 0){
            throw new BackendRuntimeException("扣券不能为零");
        }
        int leftPoint = currentUser.getPoints() - points;
        if(leftPoint < 0){
            throw new BackendRuntimeException("剩余"+currentUser.getPoints()+"券不够用");
        }
        Expenses expenses = new Expenses();
        expenses.setId(CommonUtils.generateId());
        expenses.setMemberId(currentUser.getId());
        expenses.setUniqueKey(param.getUniqueKey());
        expenses.setUniqueValue(param.getUniqueValue());
        expenses.setPoints(points);
        Date now = new Date();
        expenses.setCreatedAt(now);
        expenses.setUpdatedAt(now);
        expensesMapper.insert(expenses);
        // 真实的扣券行为

        currentUser.setPoints(leftPoint);
        currentUser.setUpdatedAt(now);
        memberService.update(currentUser);

        // 返回本次消耗券以及会员剩余券
        ExpenseVo expenseVo = new ExpenseVo();
        expenseVo.setMemberPoints(currentUser.getPoints());
        expenseVo.setPoint(param.getPoints());
        expenseVo.setUniqueKey(param.getUniqueKey());
        expenseVo.setUniqueValue(param.getUniqueValue());
        expenseVo.setCreatedAt(now);
        return expenseVo;
    }
}
