package cn.kdan.pdf.backend.core.service.impl;


import cn.kdan.pdf.backend.core.enums.ClientEnum;
import cn.kdan.pdf.backend.core.service.WXPayService;
import cn.kdan.pdf.backend.core.utils.wxpay.MyConfig;
import cn.kdan.pdf.backend.core.utils.wxpay.WXPay;
import cn.kdan.pdf.backend.core.utils.wxpay.WXPayConstants;
import cn.kdan.pdf.backend.core.utils.wxpay.WXPayOrderQueryResult;
import com.alibaba.fastjson.JSON;
import constant.CommonConstant;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @author tangxiangan
 */
@Slf4j
@Service
public class WXPayServiceImpl implements WXPayService {


    @Value("${wx.pay.notifyUrl}")
    private String notifyUrl;
    @Autowired
    private MyConfig myWXConfig;

    @Override
    public Map<String,String> unifiedOrder(String subject, String orderNo, Float price, String client) {
        WXPay wxpay = new WXPay(myWXConfig);
        Map<String, String> data = new HashMap<>();
        data.put("body", subject);
        data.put("out_trade_no", orderNo);
        //data.put("total_fee", String.valueOf(Math.round(price*100)));
        data.put("total_fee", String.valueOf(price.intValue()*100));
        //回调地址
        data.put("notify_url", notifyUrl);
        //扫码支付
        if(ClientEnum.WEB.value().equals(client)) {
            data.put("trade_type", "NATIVE");
        }else {
            data.put("trade_type", "APP");
        }
        try {
            Map<String, String> resp = wxpay.unifiedOrder(data);
            if (CommonConstant.CODE_SUCCESS.equals(resp.get(WXPayConstants.RESULT_CODE))) {
                return resp;
            } else {
                log.error(resp.get(WXPayConstants.RETURN_MSG));
                throw new BackendRuntimeException(WXPayConstants.UNIFIEDORDER_ERROR);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            throw new BackendRuntimeException(WXPayConstants.UNIFIEDORDER_ERROR);
        }
    }


    @Override
    public WXPayOrderQueryResult orderQuery(String tradeNo) {
        WXPay wxpay = new WXPay(myWXConfig);
        Map<String, String> data = new HashMap<>(1);
        //订单号
        data.put("out_trade_no", tradeNo);
        try {
            Map<String, String> resp = wxpay.orderQuery(data);
            WXPayOrderQueryResult result = JSON.parseObject(JSON.toJSONString(resp), WXPayOrderQueryResult.class);
            if (CommonConstant.CODE_SUCCESS.equals(result.getResultCode())) {
                return result;
            } else {
                log.error(resp.get(WXPayConstants.RETURN_MSG));
                throw new BackendRuntimeException(WXPayConstants.ORDER_QUERY_ERROR);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            throw new BackendRuntimeException(WXPayConstants.ORDER_QUERY_ERROR);
        }
    }

}

