package cn.kdan.pdf.backend.core.utils;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

/**
 * @author sfq
 * @description
 */
@Component
@Slf4j
public class CommonBusinessUtils {

    /**
     * 语言（zh、zh-CN、zh-Hans表示汉语，en表示英语）
     */
    private final static String ZH = "zh";
    private final static String ZH_CN = "zh-CN";
    private final static String ZH_HANS = "zh-Hans";
    private final static String EN = "en";

    private final static String[] EMAIL_REGEX = {"chuge\\..*@test.com",".*@upburv.com",".*@chacuo.net","111222@qq.com"};

    /**
     * 判断语言
     * @param language 语言类型
     * @return 语言类型
     */
    public static String judgeLanguage(String language){
        if ("".equals(language) || ZH.equals(language) || ZH_CN.equals(language) || ZH_HANS.equals(language)) {
            language = ZH_CN;
        } else {
            language = EN;
        }
        return language;
    }

    /**
     * 判断是否属于黑名单格式的邮箱，属于黑名单则返回true，不属于则返回false
     * @param email 邮箱
     * @return 是否属于黑名单邮箱或邮箱格式
     */
    public static boolean verifyBlackEmail(String email){
        if(StringUtils.isEmpty(email)){
            return false;
        }
        for (int i = 0; i < EMAIL_REGEX.length; i++) {
            boolean matches = email.matches(EMAIL_REGEX[i]);
            if(matches){
                return true;
            }
        }
        return false;
    }

    /**
     * 判断文件大小
     * @param len 文件长度
     * @param size 限制大小
     * @param unit 限制单位（B,K,M,G）
     * @return 是否结果大于
     */
    public static boolean checkFileSize(Long len, int size, String unit) {
        double fileSize = 0;
        if ("B".equals(unit.toUpperCase())) {
            fileSize = (double) len;
        } else if ("K".equals(unit.toUpperCase())) {
            fileSize = (double) len / 1024;
        } else if ("M".equals(unit.toUpperCase())) {
            fileSize = (double) len / 1048576;
        } else if ("G".equals(unit.toUpperCase())) {
            fileSize = (double) len / 1073741824;
        }
        if (fileSize > size) {
            return false;
        }
        return true;
    }

}
