package cn.kdan.pdf.backend.core.utils;

import cn.kdan.pdf.backend.core.pojo.third.Tencent;
import cn.kdan.pdf.backend.core.pojo.third.TencentMe;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.ObjectUtils;
import utils.JsonUtils;

/**
 * @author SongFuQiang
 * @description QQ登录工具类
 */
@Slf4j
public class TencentUtils {
    private static String OPENID_ENDPOINT = "https://graph.qq.com/oauth2.0/me?access_token=ACCESS_TOKEN&unionid=1";
    private static String ENDPOINT = "https://graph.qq.com/user/get_user_info?oauth_consumer_key=CLIENT_ID&openid=OPENID&access_token=ACCESS_TOKEN";

    /**
     * 获取unionId、clientId数据
     * @param accessToken 权限token
     * @return 返回unionId、clientId
     */
    public static TencentMe me(String accessToken){
        String url = OPENID_ENDPOINT.replace("ACCESS_TOKEN", accessToken);
        try {
            String tencent = HttpClientUtils.get(url);
            String trim = tencent.replace("callback(", "").replace(")", "").trim();
            TencentMe me = JsonUtils.jsonStringToBean(trim, TencentMe.class);
            if(ObjectUtils.isNotEmpty(me)) {
                if (StringUtils.isEmpty(me.getOpenid()) || StringUtils.isEmpty(me.getUnionId())) {
                    log.error("获取QQ用户权限信息失败:{},accessToken:{}", tencent, accessToken);
                    throw new BackendRuntimeException("获取QQ用户权限信息失败");
                }
                return me;
            }
        }catch (Exception e){
            log.error("获取QQ用户信息失败：{},{}",e,e.getMessage());
        }
        return null;
    }

    /**
     * 获取用户信息
     * @param accessToken 权限token
     * @return 返回qq用户信息
     */
    public static Tencent getUserInfo(String accessToken){
        Tencent tencent;
        TencentMe me = me(accessToken);
        if (ObjectUtils.isEmpty(me)) {
            throw new BackendRuntimeException("获取QQ用户权限信息失败");
        }
        String clientId = me.getClientId();
        String openid = me.getOpenid();
        String unionId = me.getUnionId();
        String url = ENDPOINT.replace("CLIENT_ID",clientId).replace("OPENID",openid).replace("ACCESS_TOKEN",accessToken);
        String resp = "";
        try{
            resp = HttpClientUtils.get(url);
            tencent = JsonUtils.jsonStringToBean(resp, Tencent.class);
        }catch (Exception e){
            log.error("获取QQ用户信息失败:{},{}",e,e.getMessage());
            throw new BackendRuntimeException("获取QQ用户信息失败");
        }
        if(ObjectUtils.isNotEmpty(tencent) && StringUtils.isBlank(tencent.getNickname())){
            log.error("获取QQ用户信息失败：{},accessToken:{}",resp,accessToken);
            throw new BackendRuntimeException("获取QQ用户信息失败");
        }
        tencent.setAccessToken(accessToken);
        tencent.setClientId(clientId);
        tencent.setUnionId(unionId);
        return tencent;
    }

    public static void main(String[] args) {
        Tencent tencent = getUserInfo("AA9EC48B21422DAA8B0F9173A0EF404A");
        System.out.println(tencent != null ? tencent.toString() : "");
    }

}
