package cn.kdan.pdf.backend.core.utils;

import cn.kdan.pdf.backend.core.pojo.third.Wechat;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.ObjectUtils;
import utils.JsonUtils;

/**
 * @author SongFuQiang
 * @description 微信登录工具类
 */
@Slf4j
public class WechatUtils {

    private static String ENDPOINT = "https://api.weixin.qq.com/sns/userinfo?openid=OPEN_ID&access_token=ACCESS_TOKEN";

    /**
     * 获取用户信息
     * @param openid openid
     * @param accessToken accessToken
     * @return 用户信息
     */
    public static Wechat getUserInfo(String openid, String accessToken){
        Wechat wechat = new Wechat();
        String url = ENDPOINT.replace("OPEN_ID",openid).replace("ACCESS_TOKEN",accessToken);
        String resp = "";
        try {
            resp = HttpClientUtils.get(url);
            wechat = JsonUtils.jsonStringToBean(resp, Wechat.class);
        }catch (Exception e){
            log.error("获取微信用户信息失败:{},token:{}", resp, accessToken);
            throw new BackendRuntimeException("获取微信用户信息失败");
        }
        if(ObjectUtils.isNotEmpty(wechat)) {
            if (StringUtils.isEmpty(wechat.getUnionId()) || StringUtils.isEmpty(wechat.getOpenid())) {
                log.error("获取微信用户信息失败:{},token:{}", resp, accessToken);
                throw new BackendRuntimeException("获取微信用户信息失败");
            }
        }
        return wechat;
    }

}
